@@ -1,7 +1,7 @@
 // samplv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -22,6 +22,7 @@
 #include "samplv1_lv2.h"
 
 #include "samplv1_sched.h"
+#include "samplv1_sample.h"
 
 #include "samplv1_programs.h"
 #include "samplv1_controls.h"
@@ -31,7 +32,12 @@
 #include "lv2/lv2plug.in/ns/ext/atom/util.h"
 
 #include "lv2/lv2plug.in/ns/ext/state/state.h"
+#include "lv2/lv2plug.in/ns/ext/patch/patch.h"
 
+#include "lv2/lv2plug.in/ns/ext/options/options.h"
+#include "lv2/lv2plug.in/ns/ext/buf-size/buf-size.h"
+
+#include <stdlib.h>
 #include <stdlib.h>
 #include <math.h>
 
@@ -40,34 +46,113 @@
 // samplv1_lv2 - impl.
 //
 
+// atom-like message used internally with worker/schedule
+typedef struct {
+	LV2_Atom atom;
+	const char *sample_path;
+} samplv1_lv2_worker_message;
+
+
 samplv1_lv2::samplv1_lv2 (
 	double sample_rate, const LV2_Feature *const *host_features )
 	: samplv1(2, float(sample_rate))
 {
 	m_urid_map = NULL;
-	m_atom_sequence = NULL;
+	m_atom_in  = NULL;
+	m_atom_out = NULL;
+	m_schedule = NULL;
+
+	m_ndelta = 0;
+
+	const LV2_Options_Option *host_options = NULL;
 
 	for (int i = 0; host_features && host_features[i]; ++i) {
-		if (::strcmp(host_features[i]->URI, LV2_URID_MAP_URI) == 0) {
-			m_urid_map = (LV2_URID_Map *) host_features[i]->data;
+		const LV2_Feature *host_feature = host_features[i];
+		if (::strcmp(host_feature->URI, LV2_URID_MAP_URI) == 0) {
+			m_urid_map = (LV2_URID_Map *) host_feature->data;
 			if (m_urid_map) {
+ 				m_urids.gen1_sample = m_urid_map->map(
+ 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_SAMPLE");
+ 				m_urids.gen1_loop_start = m_urid_map->map(
+ 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_START");
+ 				m_urids.gen1_loop_end = m_urid_map->map(
+ 					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_LOOP_END");
+				m_urids.gen1_update = m_urid_map->map(
+					m_urid_map->handle, SAMPLV1_LV2_PREFIX "GEN1_UPDATE");
 				m_urids.atom_Blank = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Blank);
 				m_urids.atom_Object = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Object);
 				m_urids.atom_Float = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Float);
+				m_urids.atom_Int = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Int);
+				m_urids.atom_Path = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__Path);
 				m_urids.time_Position = m_urid_map->map(
 					m_urid_map->handle, LV2_TIME__Position);
 				m_urids.time_beatsPerMinute = m_urid_map->map(
 					m_urid_map->handle, LV2_TIME__beatsPerMinute);
 				m_urids.midi_MidiEvent = m_urid_map->map(
 					m_urid_map->handle, LV2_MIDI__MidiEvent);
-				break;
+ 				m_urids.midi_MidiEvent = m_urid_map->map(
+ 					m_urid_map->handle, LV2_MIDI__MidiEvent);
+				m_urids.bufsz_minBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__minBlockLength);
+ 				m_urids.bufsz_maxBlockLength = m_urid_map->map(
+ 					m_urid_map->handle, LV2_BUF_SIZE__maxBlockLength);
+			#ifdef LV2_BUF_SIZE__nominalBlockLength
+				m_urids.bufsz_nominalBlockLength = m_urid_map->map(
+					m_urid_map->handle, LV2_BUF_SIZE__nominalBlockLength);
+			#endif
+				m_urids.patch_Get = m_urid_map->map(
+ 					m_urid_map->handle, LV2_PATCH__Get);
+				m_urids.patch_Set = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__Set);
+				m_urids.patch_Put = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__Put);
+				m_urids.patch_body = m_urid_map->map(
+					m_urid_map->handle, LV2_PATCH__body);
+				m_urids.patch_property = m_urid_map->map(
+ 					m_urid_map->handle, LV2_PATCH__property);
+				m_urids.patch_value = m_urid_map->map(
+ 					m_urid_map->handle, LV2_PATCH__value);
 			}
 		}
+		else
+		if (::strcmp(host_feature->URI, LV2_WORKER__schedule) == 0)
+			m_schedule = (LV2_Worker_Schedule *) host_feature->data;
+		else
+		if (::strcmp(host_feature->URI, LV2_OPTIONS__options) == 0)
+			host_options = (const LV2_Options_Option *) host_feature->data;
 	}
 
+	uint32_t buffer_size = 0; // whatever happened to safe default?
+
+	for (int i = 0; host_options && host_options[i].key; ++i) {
+		const LV2_Options_Option *host_option = &host_options[i];
+		if (host_option->type == m_urids.atom_Int) {
+			uint32_t block_length = 0;
+			if (host_option->key == m_urids.bufsz_minBlockLength)
+				block_length = *(int *) host_option->value;
+			else
+			if (host_option->key == m_urids.bufsz_maxBlockLength)
+				block_length = *(int *) host_option->value;
+		#ifdef LV2_BUF_SIZE__nominalBlockLength
+			else
+			if (host_option->key == m_urids.bufsz_nominalBlockLength)
+				block_length = *(int *) host_option->value;
+		#endif
+			// choose the lengthier...
+			if (buffer_size < block_length)
+				buffer_size = block_length;
+		}
+ 	}
+ 
+	samplv1::setBufferSize(buffer_size);
+
+	lv2_atom_forge_init(&m_forge, m_urid_map);
+
 	const uint16_t nchannels = samplv1::channels();
 	m_ins  = new float * [nchannels];
 	m_outs = new float * [nchannels];
@@ -90,7 +175,10 @@ void samplv1_lv2::connect_port ( uint32_t port, void *data )
 {
 	switch(PortIndex(port)) {
 	case MidiIn:
-		m_atom_sequence = (LV2_Atom_Sequence *) data;
+		m_atom_in = (LV2_Atom_Sequence *) data;
+		break;
+	case Notify:
+		m_atom_out = (LV2_Atom_Sequence *) data;
 		break;
 	case AudioInL:
 		m_ins[0] = (float *) data;
@@ -120,20 +208,28 @@ void samplv1_lv2::run ( uint32_t nframes )
 		outs[k] = m_outs[k];
 	}
 
+	if (m_atom_out) {
+		const uint32_t capacity = m_atom_out->atom.size;
+		lv2_atom_forge_set_buffer(&m_forge, (uint8_t *) m_atom_out, capacity);
+		lv2_atom_forge_sequence_head(&m_forge, &m_notify_frame, 0);
+	}
+
 	uint32_t ndelta = 0;
 
-	if (m_atom_sequence) {
-		LV2_ATOM_SEQUENCE_FOREACH(m_atom_sequence, event) {
+	if (m_atom_in) {
+		LV2_ATOM_SEQUENCE_FOREACH(m_atom_in, event) {
 			if (event == NULL)
 				continue;
 			if (event->body.type == m_urids.midi_MidiEvent) {
 				uint8_t *data = (uint8_t *) LV2_ATOM_BODY(&event->body);
-				const uint32_t nread = event->time.frames - ndelta;
-				if (nread > 0) {
-					samplv1::process(ins, outs, nread);
-					for (uint16_t k = 0; k < nchannels; ++k) {
-						ins[k]  += nread;
-						outs[k] += nread;
+				if (event->time.frames > ndelta) {
+					const uint32_t nread = event->time.frames - ndelta;
+					if (nread > 0) {
+						samplv1::process(ins, outs, nread);
+						for (uint16_t k = 0; k < nchannels; ++k) {
+							ins[k]  += nread;
+							outs[k] += nread;
+						}
 					}
 				}
 				ndelta = event->time.frames;
@@ -149,23 +245,77 @@ void samplv1_lv2::run ( uint32_t nframes )
 					lv2_atom_object_get(object,
 						m_urids.time_beatsPerMinute, &atom, NULL);
 					if (atom && atom->type == m_urids.atom_Float) {
-						const float bpm_sync = samplv1::paramValue(samplv1::DEL1_BPMSYNC);
-						if (bpm_sync > 0.0f) {
-							const float bpm_host = samplv1::paramValue(samplv1::DEL1_BPMHOST);
-							if (bpm_host > 0.0f) {
-								const float bpm	= ((LV2_Atom_Float *) atom)->body;
-								if (::fabs(bpm_host - bpm) > 0.01f)
-									samplv1::setParamValue(samplv1::DEL1_BPMHOST, bpm);
+						const float host_bpm = ((LV2_Atom_Float *) atom)->body;
+						if (::fabsf(host_bpm - samplv1::tempo()) > 0.001f)
+							samplv1::setTempo(host_bpm);
+					}
+				}
+				else 
+				if (object->body.otype == m_urids.patch_Set) {
+					// set property value
+					const LV2_Atom *property = NULL;
+					const LV2_Atom *value = NULL;
+					lv2_atom_object_get(object,
+						m_urids.patch_property, &property,
+						m_urids.patch_value, &value, 0);
+					if (property && value && property->type == m_forge.URID) {
+						const uint32_t key = ((const LV2_Atom_URID *) property)->body;
+						const LV2_URID type = value->type;
+						if (key == m_urids.gen1_sample
+							&& type == m_urids.atom_Path) {
+							if (m_schedule) {
+								samplv1_lv2_worker_message mesg;
+								mesg.atom.type = key;
+								mesg.atom.size = sizeof(mesg.sample_path);
+								mesg.sample_path
+									= (const char *) LV2_ATOM_BODY_CONST(value);
+								// schedule loading new sample
+								m_schedule->schedule_work(
+									m_schedule->handle, sizeof(mesg), &mesg);
+							}
+						}
+						else
+						if (key == m_urids.gen1_loop_start
+							&& type == m_urids.atom_Int) {
+							samplv1_sample *pSample = samplv1::sample();
+							if (pSample) {
+								const uint32_t loop_start
+									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+								const uint32_t loop_end = pSample->loopEnd();
+								setLoopRange(loop_start, loop_end);
+							}
+						}
+						else
+						if (key == m_urids.gen1_loop_end
+							&& type == m_urids.atom_Int) {
+							samplv1_sample *pSample = samplv1::sample();
+							if (pSample) {
+								const uint32_t loop_start = pSample->loopStart();
+								const uint32_t loop_end
+									= *(uint32_t *) LV2_ATOM_BODY_CONST(value);
+								setLoopRange(loop_start, loop_end);
 							}
 						}
 					}
 				}
+				else
+				if (object->body.otype == m_urids.patch_Get) {
+					// put property values (probably to UI)
+					patch_put(ndelta);
+				}
 			}
 		}
-	//	m_atom_sequence = NULL;
+		// remember last time for worker response
+		m_ndelta = ndelta;
+	//	m_atom_in = NULL;
 	}
 
-	samplv1::process(ins, outs, nframes - ndelta);
+	if (nframes > ndelta)
+		samplv1::process(ins, outs, nframes - ndelta);
+
+	// test for current sample loop change
+	if (samplv1::sampleLoopTest())
+		updateSample();
 }
 
 
@@ -342,6 +492,8 @@ static LV2_State_Status samplv1_lv2_state_restore ( LV2_Handle instance,
 	if (loop_start < loop_end)
 		pPlugin->setLoopRange(loop_start, loop_end);
 
+	pPlugin->reset();
+
 	samplv1_sched::sync_notify(pPlugin, samplv1_sched::Sample, 1);
 
 	return LV2_STATE_SUCCESS;
@@ -393,6 +545,77 @@ void samplv1_lv2::select_program ( uint32_t bank, uint32_t program )
 #endif	// CONFIG_LV2_PROGRAMS
 
 
+void samplv1_lv2::updateSample (void)
+{
+	if (m_schedule) {
+		samplv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.gen1_update;
+		mesg.atom.size = sizeof(mesg.sample_path);
+		mesg.sample_path = samplv1::sampleFile();
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+}
+
+
+bool samplv1_lv2::patch_put ( uint32_t ndelta )
+{
+	static char s_szNull[1] = {'\0'};
+	const char *pszSampleFile = NULL;
+	samplv1_sample *pSample = samplv1::sample();
+	if (pSample)
+		pszSampleFile = pSample->filename();
+	if (pszSampleFile == NULL)
+		pszSampleFile = s_szNull;
+
+	lv2_atom_forge_frame_time(&m_forge, ndelta);
+
+	LV2_Atom_Forge_Frame patch_frame;
+	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Put);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_body);
+
+	LV2_Atom_Forge_Frame body_frame;
+	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
+	lv2_atom_forge_key(&m_forge, m_urids.gen1_sample);
+	lv2_atom_forge_path(&m_forge, pszSampleFile, ::strlen(pszSampleFile) + 1);
+	lv2_atom_forge_key(&m_forge, m_urids.gen1_loop_start);
+	lv2_atom_forge_int(&m_forge, pSample->loopStart());
+	lv2_atom_forge_key(&m_forge, m_urids.gen1_loop_end);
+	lv2_atom_forge_int(&m_forge, pSample->loopEnd());
+
+	lv2_atom_forge_pop(&m_forge, &body_frame);
+	lv2_atom_forge_pop(&m_forge, &patch_frame);
+
+	return true;
+}
+
+
+bool samplv1_lv2::worker_work ( const void *data, uint32_t /*size*/ )
+{
+	const samplv1_lv2_worker_message *mesg
+		= (const samplv1_lv2_worker_message *) data;
+
+	if (mesg->atom.type == m_urids.gen1_update)
+		return true;
+	else
+	if (mesg->atom.type == m_urids.gen1_sample) {
+		samplv1::setSampleFile(mesg->sample_path);
+		return true;
+	}
+
+	return false;
+}
+
+
+bool samplv1_lv2::worker_response ( const void */*data*/, uint32_t /*size*/ )
+{
+	// update all properties, and eventually, any observers...
+	samplv1_sched::sync_notify(this, samplv1_sched::Sample, 0);
+
+	return patch_put(m_ndelta);
+}
+
+
 //-------------------------------------------------------------------------
 // samplv1_lv2 - LV2 desc.
 //
@@ -474,17 +697,54 @@ static const LV2_Programs_Interface samplv1_lv2_programs_interface =
 
 #endif	// CONFIG_LV2_PROGRAMS
 
+
+static LV2_Worker_Status samplv1_lv2_worker_work (
+	LV2_Handle instance, LV2_Worker_Respond_Function respond,
+	LV2_Worker_Respond_Handle handle, uint32_t size, const void *data )
+{
+	samplv1_lv2 *pSampl = static_cast<samplv1_lv2 *> (instance);
+	if (pSampl && pSampl->worker_work(data, size)) {
+		respond(handle, size, data);
+		return LV2_WORKER_SUCCESS;
+	}
+
+	return LV2_WORKER_ERR_UNKNOWN;
+}
+
+
+static LV2_Worker_Status samplv1_lv2_worker_response (
+	LV2_Handle instance, uint32_t size, const void *data )
+{
+	samplv1_lv2 *pSampl = static_cast<samplv1_lv2 *> (instance);
+	if (pSampl && pSampl->worker_response(data, size))
+		return LV2_WORKER_SUCCESS;
+	else
+		return LV2_WORKER_SUCCESS;
+}
+
+
+static const LV2_Worker_Interface samplv1_lv2_worker_interface =
+{
+	samplv1_lv2_worker_work,
+	samplv1_lv2_worker_response,
+	NULL
+};
+
+
 static const void *samplv1_lv2_extension_data ( const char *uri )
 {
 #ifdef CONFIG_LV2_PROGRAMS
 	if (::strcmp(uri, LV2_PROGRAMS__Interface) == 0)
-		return (void *) &samplv1_lv2_programs_interface;
+		return &samplv1_lv2_programs_interface;
 	else
 #endif
-	if (::strcmp(uri, LV2_STATE__interface))
-		return NULL;
+	if (::strcmp(uri, LV2_WORKER__interface) == 0)
+		return &samplv1_lv2_worker_interface;
+	else
+	if (::strcmp(uri, LV2_STATE__interface) == 0)
+		return &samplv1_lv2_state_interface;
 
-	return &samplv1_lv2_state_interface;
+	return NULL;
 }
 
 
