@@ -1,7 +1,7 @@
 // samplv1widget.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2015, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2016, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -129,6 +129,8 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QStringList slopes;
 	slopes << tr("12dB/oct");
 	slopes << tr("24dB/oct");
+	slopes << tr("Biquad");
+	slopes << tr("Formant");
 
 	m_ui.Dcf1SlopeKnob->insertItems(0, slopes);
 
@@ -140,11 +142,14 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Gen1ReverseKnob->insertItems(0, states);
 	m_ui.Gen1LoopKnob->insertItems(0, states);
 
+	m_ui.Lfo1SyncKnob->insertItems(0, states);
+
 	m_ui.Dyn1CompressKnob->insertItems(0, states);
 	m_ui.Dyn1LimiterKnob->insertItems(0, states);
 
 	// Special values
 	const QString& sOff = states.first();
+	m_ui.Gen1GlideKnob->setSpecialValueText(sOff);
 	m_ui.Cho1WetKnob->setSpecialValueText(sOff);
 	m_ui.Fla1WetKnob->setSpecialValueText(sOff);
 	m_ui.Pha1WetKnob->setSpecialValueText(sOff);
@@ -153,6 +158,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	const QString& sAuto = tr("Auto");
 	m_ui.Gen1EnvTimeKnob->setSpecialValueText(sAuto);
+	m_ui.Lfo1BpmKnob->setSpecialValueText(sAuto);
 	m_ui.Del1BpmKnob->setSpecialValueText(sAuto);
 
 	// Wave integer widths.
@@ -175,6 +181,10 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Dcf1EnvelopeKnob->setMaximum(+1.0f);
 
 	// LFO parameter limits.
+	m_ui.Lfo1BpmKnob->setScale(1.0f);
+	m_ui.Lfo1BpmKnob->setMinimum(0.0f);
+	m_ui.Lfo1BpmKnob->setMaximum(360.0f);
+	m_ui.Lfo1BpmKnob->setSingleStep(1.0f);
 	m_ui.Lfo1SweepKnob->setMinimum(-1.0f);
 	m_ui.Lfo1SweepKnob->setMaximum(+1.0f);
 	m_ui.Lfo1CutoffKnob->setMinimum(-1.0f);
@@ -209,7 +219,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 
 	// Effects (delay BPM)
 	m_ui.Del1BpmKnob->setScale(1.0f);
-	m_ui.Del1BpmKnob->setMinimum(3.6f);
+	m_ui.Del1BpmKnob->setMinimum(0.0f);
 	m_ui.Del1BpmKnob->setMaximum(360.0f);
 	m_ui.Del1BpmKnob->setSingleStep(1.0f);
 
@@ -289,7 +299,9 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// LFO1
 	setParamKnob(samplv1::LFO1_SHAPE,   m_ui.Lfo1ShapeKnob);
 	setParamKnob(samplv1::LFO1_WIDTH,   m_ui.Lfo1WidthKnob);
+	setParamKnob(samplv1::LFO1_BPM,     m_ui.Lfo1BpmKnob);
 	setParamKnob(samplv1::LFO1_RATE,    m_ui.Lfo1RateKnob);
+	setParamKnob(samplv1::LFO1_SYNC,    m_ui.Lfo1SyncKnob);
 	setParamKnob(samplv1::LFO1_PANNING, m_ui.Lfo1PanningKnob);
 	setParamKnob(samplv1::LFO1_VOLUME,  m_ui.Lfo1VolumeKnob);
 	setParamKnob(samplv1::LFO1_CUTOFF,  m_ui.Lfo1CutoffKnob);
@@ -388,6 +400,7 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	// OUT1
 	setParamKnob(samplv1::OUT1_WIDTH,   m_ui.Out1WidthKnob);
 	setParamKnob(samplv1::OUT1_PANNING, m_ui.Out1PanningKnob);
+	setParamKnob(samplv1::OUT1_FXSEND,  m_ui.Out1FxSendKnob);
 	setParamKnob(samplv1::OUT1_VOLUME,  m_ui.Out1VolumeKnob);
 
 
@@ -414,10 +427,6 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(samplv1::DEL1_FEEDB, m_ui.Del1FeedbKnob);
 	setParamKnob(samplv1::DEL1_BPM,   m_ui.Del1BpmKnob);
 
-	QObject::connect(m_ui.Del1BpmKnob,
-		SIGNAL(valueChanged(float)),
-		SLOT(bpmSyncChanged()));
-
 	// Reverb
 	setParamKnob(samplv1::REV1_WET,   m_ui.Rev1WetKnob);
 	setParamKnob(samplv1::REV1_ROOM,  m_ui.Rev1RoomKnob);
@@ -628,20 +637,15 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 		break;
 	}
 #endif
+	case samplv1::DCF1_SLOPE:
+		m_ui.Dcf1TypeKnob->setEnabled(int(fValue) != 3); // !Formant
+		break;
 	case samplv1::GEN1_LOOP: {
 		const bool bLoop = bool(fValue > 0.0f);
 		pSamplUi->setLoop(bLoop);
-		m_ui.Gen1Sample->setLoop(bLoop);
-		m_ui.Gen1Sample->setLoopStart(pSamplUi->loopStart());
-		m_ui.Gen1Sample->setLoopEnd(pSamplUi->loopEnd());
-		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
 		updateSampleLoop(pSamplUi->sample());
 		break;
 	}
-	case samplv1::DEL1_BPMSYNC:
-		if (fValue > 0.0f)
-			m_ui.Del1BpmKnob->setValue(0.0f);
-		// Fall thru...
 	default:
 		break;
 	}
@@ -689,7 +693,7 @@ void samplv1widget::resetParams (void)
 			fValue = pKnob->defaultValue();
 		setParamValue(index, fValue);
 		updateParam(index, fValue);
-		m_params_ab[index] = fValue;
+		m_params_ab[i] = fValue;
 	}
 
 	m_ui.StatusBar->showMessage(tr("Reset preset"), 5000);
@@ -713,10 +717,10 @@ void samplv1widget::swapParams ( bool bOn )
 		samplv1widget_knob *pKnob = paramKnob(index);
 		if (pKnob) {
 			const float fOldValue = pKnob->value();
-			const float fNewValue = m_params_ab[index];
+			const float fNewValue = m_params_ab[i];
 			setParamValue(index, fNewValue);
 			updateParam(index, fNewValue);
-			m_params_ab[index] = fOldValue;
+			m_params_ab[i] = fOldValue;
 		}
 	}
 
@@ -750,7 +754,7 @@ void samplv1widget::updateParamValues (void)
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
 	//	updateParamEx(index, fValue);
-		m_params_ab[index] = fValue;
+		m_params_ab[i] = fValue;
 	}
 }
 
@@ -765,7 +769,8 @@ void samplv1widget::resetParamValues (void)
 		const float fValue = samplv1_param::paramDefaultValue(index);
 		setParamValue(index, fValue, true);
 		updateParam(index, fValue);
-		m_params_ab[index] = fValue;
+	//	updateParamEx(index, fValue);
+		m_params_ab[i] = fValue;
 	}
 }
 
@@ -1013,14 +1018,17 @@ void samplv1widget::loopEndChanged (void)
 void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 {
 	if (pSample) {
+		const bool bLoop = pSample->isLoop();
+		m_ui.Gen1Sample->setLoop(bLoop);
+		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
 		const uint32_t iLoopStart = pSample->loopStart();
 		const uint32_t iLoopEnd = pSample->loopEnd();
 		const uint32_t nframes = pSample->length();
 		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
 		m_ui.Gen1LoopStartSpinBox->setMaximum(
-			iLoopEnd > 0 ? iLoopEnd - 1 : 0);
+		    iLoopEnd > 0 ? iLoopEnd : 0);
 		m_ui.Gen1LoopEndSpinBox->setMinimum(
-			iLoopStart < nframes ? iLoopStart + 1 : nframes);
+		    iLoopStart < nframes ? iLoopStart : nframes);
 		m_ui.Gen1LoopEndSpinBox->setMaximum(nframes);
 		m_ui.Gen1LoopStartSpinBox->setValue(iLoopStart);
 		m_ui.Gen1LoopEndSpinBox->setValue(iLoopEnd);
@@ -1030,6 +1038,8 @@ void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 			updateDirtyPreset(true);
 		}
 	} else {
+		m_ui.Gen1Sample->setLoop(false);
+		m_ui.Gen1LoopRangeFrame->setEnabled(false);
 		m_ui.Gen1LoopStartSpinBox->setMinimum(0);
 		m_ui.Gen1LoopStartSpinBox->setMaximum(0);
 		m_ui.Gen1LoopStartSpinBox->setValue(0);
@@ -1064,24 +1074,6 @@ void samplv1widget::activateParamKnobsGroupBox (
 }
 
 
-// Delay BPM change.
-void samplv1widget::bpmSyncChanged (void)
-{
-	if (m_iUpdate > 0)
-		return;
-
-	++m_iUpdate;
-	samplv1_ui *pSamplUi = ui_instance();
-	if (pSamplUi) {
-		const bool bBpmSync0 = (pSamplUi->paramValue(samplv1::DEL1_BPMSYNC) > 0.0f);
-		const bool bBpmSync1 = m_ui.Del1BpmKnob->isSpecialValue();
-		if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
-			pSamplUi->setParamValue(samplv1::DEL1_BPMSYNC, (bBpmSync1 ? 1.0f : 0.0f));
-	}
-	--m_iUpdate;
-}
-
-
 // Sample context menu.
 void samplv1widget::contextMenuRequest ( const QPoint& pos )
 {
